@@ -25,8 +25,8 @@
  */
 
 var util    = require( './util' ),
-    warn    = require( './warn' ),
-    Warning = warn.Warning,
+    Warning = require( './warn' ).Warning,
+    Symbol  = require( './util/Symbol' ),
 
     hasOwn = Object.prototype.hasOwnProperty,
 
@@ -72,9 +72,21 @@ var util    = require( './util' ),
      */
     public_methods = {
         '__construct': true,
+        '__mixin':     true,
         'toString':    true,
         '__toString':  true,
-    };
+    },
+
+    /**
+     * Symbol used to encapsulate internal data
+     *
+     * Note that this is intentionally generated *outside* the ClassBuilder
+     * instance; this ensures that it is properly encapsulated and will not
+     * be exposed on the Classbuilder instance (which would defeat the
+     * purpose).
+     */
+    _priv = Symbol()
+;
 
 
 /**
@@ -90,15 +102,23 @@ var util    = require( './util' ),
  * @constructor
  */
 module.exports = exports =
-function ClassBuilder( member_builder, visibility_factory )
+function ClassBuilder( warn_handler, member_builder, visibility_factory )
 {
     // allow ommitting the 'new' keyword
     if ( !( this instanceof exports ) )
     {
         // module.exports for Closure Compiler
-        return new module.exports( member_builder, visibility_factory );
+        return new module.exports(
+            warn_handler, member_builder, visibility_factory
+        );
     }
 
+    /**
+     * Determines how warnings should be handled
+     * @type {WarningHandler}
+     */
+    this._warnHandler = warn_handler;
+
     /**
      * Used for building class members
      * @type {Object}
@@ -214,11 +234,11 @@ exports.getForcedPublicMethods = function()
  *
  * @param  {Function|Object}  cls  class from which to retrieve metadata
  *
- * @return  {__class_meta}
+ * @return  {__class_meta} or null if unavailable
  */
 exports.getMeta = function( cls )
 {
-    return cls.___$$meta$$ || {};
+    return ( cls[ _priv ] || {} ).meta || null;
 }
 
 
@@ -319,12 +339,14 @@ exports.prototype.build = function extend( _, __ )
             props:   this._memberBuilder.initMembers(),
         },
 
+        meta = exports.getMeta( base ) || {},
+
         abstract_methods =
-            util.clone( exports.getMeta( base ).abstractMethods )
-            || { __length: 0 }
+            util.clone( meta.abstractMethods )
+            || { __length: 0 },
 
         virtual_members =
-            util.clone( exports.getMeta( base ).virtualMembers )
+            util.clone( meta.virtualMembers )
             || {}
     ;
 
@@ -333,7 +355,7 @@ exports.prototype.build = function extend( _, __ )
     {
         throw Error(
             "Cannot extend final class " +
-                ( base.___$$meta$$.name || '(anonymous)' )
+                ( base[ _priv ].meta.name || '(anonymous)' )
         );
     }
 
@@ -365,7 +387,8 @@ exports.prototype.build = function extend( _, __ )
     // if we are inheriting from a prototype, we must make sure that all
     // properties initialized by the ctor are implicitly public; otherwise,
     // proxying will fail to take place
-    if ( !( prototype instanceof exports.ClassBase ) )
+    // TODO: see Class.isA TODO
+    if ( ( prototype[ _priv ] || {} ).vis === undefined )
     {
         this._discoverProtoProps( prototype, prop_init );
     }
@@ -395,7 +418,7 @@ exports.prototype.build = function extend( _, __ )
         // intercept warnings /only/
         if ( e instanceof Warning )
         {
-            warn.handle( e );
+            this._warnHandler.handle( e );
         }
         else
         {
@@ -410,10 +433,12 @@ exports.prototype.build = function extend( _, __ )
     var new_class = this.createCtor( cname, abstract_methods, members );
 
     // closure to hold static initialization to be used later by subtypes
-    initStaticVisibilityObj( new_class );
+    this.initStaticVisibilityObj( new_class );
+
+    var _self = this;
     var staticInit = function( ctor, inheriting )
     {
-        attachStatic( ctor, static_members, base, inheriting );
+        _self.attachStatic( ctor, static_members, base, inheriting );
     }
     staticInit( new_class, false );
 
@@ -577,7 +602,8 @@ exports.prototype.buildMembers = function buildMembers(
         var parser = props.___$$parser$$;
         delete props.___$$parser$$;
 
-        function hjoin( name, orig )
+        // TODO: this is recreated every call!
+        var hjoin = function( name, orig )
         {
             handlers[ name ] = function()
             {
@@ -592,7 +618,7 @@ exports.prototype.buildMembers = function buildMembers(
                 args.push( orig );
                 parser[ name ].apply( context, args );
             };
-        }
+        };
 
         // this avoids a performance penalty unless the above property is
         // set
@@ -786,16 +812,19 @@ function validateAbstract( ctor, cname, abstract_methods, auto )
  */
 exports.prototype.createCtor = function( cname, abstract_methods, members )
 {
-    // concrete class
+    var new_class;
+
     if ( abstract_methods.__length === 0 )
     {
-        return this.createConcreteCtor( cname, members );
+        new_class = this.createConcreteCtor( cname, members );
     }
-    // abstract class
     else
     {
-        return this.createAbstractCtor( cname );
+        new_class = this.createAbstractCtor( cname );
     }
+
+    util.defineSecureProp( new_class, _priv, {} );
+    return new_class;
 }
 
 
@@ -851,10 +880,17 @@ exports.prototype.createConcreteCtor = function( cname, members )
         // generate and store unique instance id
         attachInstanceId( this, ++_self._instanceId );
 
-        // handle internal trait initialization logic, if provided
-        if ( typeof this.___$$tctor$$ === 'function' )
+        // FIXME: this is a bit of a kluge for determining whether the ctor
+        // should be invoked before a child prector...
+        var haspre = ( typeof this.___$$ctor$pre$$ === 'function' );
+        if ( haspre
+            && ClassInstance.prototype.hasOwnProperty( '___$$ctor$pre$$' )
+        )
         {
-            this.___$$tctor$$.call( this );
+            // FIXME: we're exposing _priv to something that can be
+            // malicously set by the user
+            this.___$$ctor$pre$$( _priv );
+            haspre = false;
         }
 
         // call the constructor, if one was provided
@@ -866,6 +902,17 @@ exports.prototype.createConcreteCtor = function( cname, members )
             this.__construct.apply( this, ( args || arguments ) );
         }
 
+        // FIXME: see above
+        if ( haspre )
+        {
+            this.___$$ctor$pre$$( _priv );
+        }
+
+        if ( typeof this.___$$ctor$post$$ === 'function' )
+        {
+            this.___$$ctor$post$$( _priv );
+        }
+
         args = null;
 
         // attach any instance properties/methods (done after
@@ -978,7 +1025,8 @@ exports.prototype._attachPropInit = function(
         inherit = !!inherit;
 
         var iid    = this.__iid,
-            parent = prototype.___$$parent$$;
+            parent = prototype.___$$parent$$,
+            vis    = this[ _priv ].vis;
 
         // first initialize the parent's properties, so that ours will overwrite
         // them
@@ -994,7 +1042,7 @@ exports.prototype._attachPropInit = function(
         // this will return our property proxy, if supported by our environment,
         // otherwise just a normal object with everything merged in
         var inst_props = _self._visFactory.createPropProxy(
-            this, this.___$$vis$$, properties[ 'public' ]
+            this, vis, properties[ 'public' ]
         );
 
         // Copies all public and protected members into inst_props and stores
@@ -1002,7 +1050,7 @@ exports.prototype._attachPropInit = function(
         // chain and is returned. This is stored in a property referenced by the
         // class id, so that the private members can be swapped on each method
         // request, depending on calling context.
-        var vis = this.___$$vis$$[ cid ] = _self._visFactory.setup(
+        var vis = vis[ cid ] = _self._visFactory.setup(
             inst_props, properties, members
         );
 
@@ -1043,7 +1091,7 @@ function keywordStatic( keywords )
  *
  * @return  {undefined}
  */
-function initStaticVisibilityObj( ctor )
+exports.prototype.initStaticVisibilityObj = function( ctor )
 {
     var _self = this;
 
@@ -1095,7 +1143,7 @@ function initStaticVisibilityObj( ctor )
  *
  * @return  {undefined}
  */
-function attachStatic( ctor, members, base, inheriting )
+exports.prototype.attachStatic = function( ctor, members, base, inheriting )
 {
     var methods = members.methods,
         props   = members.props,
@@ -1226,21 +1274,13 @@ function createMeta( func, cparent )
     // copy the parent prototype's metadata if it exists (inherit metadata)
     if ( parent_meta )
     {
-        func.___$$meta$$ = util.clone( parent_meta, true );
-    }
-    else
-    {
-        // create empty
-        func.___$$meta$$ = {
-            implemented: [],
-        };
+        return func[ _priv ].meta = util.clone( parent_meta, true );
     }
 
-    // store the metadata in the prototype as well (inconsiderable overhead;
-    // it's just a reference)
-    func.prototype.___$$meta$$ = func.___$$meta$$;
-
-    return func.___$$meta$$;
+    // create empty
+    return func[ _priv ].meta = {
+        implemented: [],
+    };
 }
 
 
@@ -1291,8 +1331,12 @@ function initInstance( instance )
     var prot = function() {};
     prot.prototype = instance;
 
+    // initialize our *own* private metadata store; do not use the
+    // prototype's
+    util.defineSecureProp( instance, _priv, {} );
+
     // add the visibility objects to the data object for this class instance
-    instance.___$$vis$$ = new prot();
+    instance[ _priv ].vis = new prot();
 }
 
 
@@ -1340,10 +1384,16 @@ function attachInstanceOf( instance )
  */
 exports.getMethodInstance = function( inst, cid )
 {
-    var iid  = inst.__iid,
-        data = inst.___$$vis$$;
+    if ( inst === undefined )
+    {
+        return null;
+    }
 
-    return ( iid && data )
+    var iid  = inst.__iid,
+        priv = inst[ _priv ],
+        data;
+
+    return ( iid && priv && ( data = priv.vis ) )
         ? data[ cid ]
         : null
     ;
