@@ -41,7 +41,7 @@ module.exports = exports = function FallbackVisibilityObjectFactory()
 /**
  * "Inherit" from VisibilityObjectFactory
  */
-exports.prototype = require( __dirname + '/VisibilityObjectFactory' )();
+exports.prototype = require( './VisibilityObjectFactory' )();
 
 
 /**
