@@ -176,6 +176,9 @@ Trait.extend = function( dfn )
         return ''+name;
     };
 
+    // we're not a param trait, but we want consistent data
+    Trait.___$$mixinargs = [];
+
     // invoked to trigger mixin
     Trait.__mixin = function( dfn, tc, base )
     {
