@@ -500,7 +500,7 @@ function _isInstanceOf( type, instance )
     var meta;
     if ( !instance.__cid || !( meta = ClassBuilder.getMeta( proto ) ) )
     {
-        return false;
+        return isCompat( type, instance );
     }
 
     var implemented = meta.implemented,
