@@ -82,6 +82,8 @@ require( 'common' ).testCase(
     /**
      * A trait can only be used by something else---it does not make sense
      * to instantiate them directly, since they form an incomplete picture.
+     *
+     * Now, that said, see parameterized traits.
      */
     '@each(ctor) Cannot instantiate trait without error': function( T )
     {
@@ -127,8 +129,8 @@ require( 'common' ).testCase(
     '@each(ctor) Supertype definition is applied when using traits':
     function( T )
     {
-        var expected  = 'bar';
-            expected2 = 'baz';
+        var expected  = 'bar',
+            expected2 = 'baz',
             Foo       = this.Class( { foo: expected } ),
             SubFoo    = this.Class.use( T( {} ) )
                 .extend( Foo, { bar: expected2  } );
@@ -210,7 +212,9 @@ require( 'common' ).testCase(
             return;
         }
 
-        this.fail( "Traits should not be able to define __construct" );
+        this.fail( false, true,
+            "Traits should not be able to define __construct"
+        );
     },
 
 
